package board;

import java.util.*;
import java.awt.*;
import javax.swing.*;
public class Doll {
	Vector2D position;
	Limb root_limb;

	Doll(){
		root_limb = new Limb(new Vector2D(0,0), -Math.PI/2, 80, Limb.BODY);
		Limb head = new Limb(new Vector2D(90,0), 0, 30, Limb.HEAD);
		Limb left_upper_arm = new Limb(new Vector2D(80,30), -Math.PI*7/8, 50, Limb.LIMB);
		Limb right_upper_arm = new Limb(new Vector2D(80,-30), Math.PI*7/8, 50, Limb.LIMB);
		Limb left_lower_arm = new Limb(new Vector2D(50,0), -Math.PI/8, 50, Limb.LIMB);
		Limb right_lower_arm = new Limb(new Vector2D(50,0), Math.PI/8, 50, Limb.LIMB);
		Limb left_upper_leg = new Limb(new Vector2D(0,0), Math.PI + Math.PI/8, 60, Limb.LIMB);
		Limb right_upper_leg = new Limb(new Vector2D(0,0), Math.PI-Math.PI/8, 60, Limb.LIMB);
		Limb left_lower_leg = new Limb(new Vector2D(60,0), -Math.PI/8, 50, Limb.LIMB);
		Limb right_lower_leg = new Limb(new Vector2D(60,0), Math.PI/8, 50, Limb.LIMB);
		
		root_limb.children.add(head);
		root_limb.children.add(left_upper_arm);
		root_limb.children.add(right_upper_arm);
		left_upper_arm.children.add(left_lower_arm);
		right_upper_arm.children.add(right_lower_arm);

		root_limb.children.add(left_upper_leg);
		root_limb.children.add(right_upper_leg);
		left_upper_leg.children.add(left_lower_leg);
		right_upper_leg.children.add(right_lower_leg);
		
		position = new Vector2D(300,300);
	}
	
	public void paint(Graphics g){
		g.setColor(Color.black);
		root_limb.paint(g, position, new Vector2D(1,0));
	}
	public ArrayList<Limb> get_all_limbs(){
		ArrayList<Limb> limbs = new ArrayList();
		get_all_limbs(root_limb, limbs);
		return limbs;
	}
	public void get_all_limbs(Limb parent, ArrayList<Limb> limbs){
		limbs.add(parent);
		for(Limb child : parent.children){
			get_all_limbs(child, limbs);
		}
	}
}

class Limb {
	final Vector2D relative_position;
	final double length;
	final double rest_angle;	
	double relative_angle;
	final ArrayList<Limb> children = new ArrayList(); 
	final int type;
	static final int BODY = 0;
	static final int HEAD = 1;
	static final int LIMB = 2;
	
	
	Limb(Vector2D relative_position, double relative_angle, double length, int type){
		this.relative_position = relative_position;
		this.rest_angle = relative_angle;
		this.relative_angle = 0;
		this.length = length;
		this.type =type;
	}
	public void paint(Graphics g, Vector2D parent_position, Vector2D parent_axis){
		
		Vector2D absolute_position = coord_map(relative_position, parent_position, parent_axis);
		Vector2D absolute_vec = Vector2D.rotate(parent_axis,  rest_angle + relative_angle);
		
		Vector2D absolute_end_position = Vector2D.add(absolute_position, Vector2D.multiply(absolute_vec, length));
		
		if (type == LIMB)
			g.drawLine((int)absolute_position.x, (int)absolute_position.y, (int)absolute_end_position.x, (int)absolute_end_position.y);
		else if (type == HEAD){
			double r = 0.5*Vector2D.distance(absolute_position, absolute_end_position);
			Vector2D center = Vector2D.mid_point(absolute_position, absolute_end_position);
			g.drawOval((int)(center.x-r), (int)(center.y-r), (int)(r*2), (int)(r*2));
		}
		else if (type == BODY){
			Vector2D left = coord_map(new Vector2D(80,30), absolute_position, absolute_vec);
			Vector2D right = coord_map(new Vector2D(80,-30), absolute_position,  absolute_vec);
			g.drawLine((int)absolute_position.x, (int)absolute_position.y, (int) left.x, (int)left.y);
			g.drawLine((int) left.x, (int)left.y, (int) right.x, (int)right.y);
			g.drawLine((int) right.x, (int)right.y, (int)absolute_position.x, (int)absolute_position.y);
		}
		
		for(Limb child : children){
			child.paint(g, absolute_position,  absolute_vec);
		}
	}
	Vector2D coord_map(Vector2D relative_position, Vector2D parent_position, Vector2D x_vec){
		Vector2D y_vec = Vector2D.rotate90(x_vec);
		Vector2D absolute_position = Vector2D.add(parent_position, Vector2D.add(Vector2D.multiply(x_vec,  relative_position.x), Vector2D.multiply(y_vec,  relative_position.y)));
		return absolute_position;
	}
	

}

