package board;

import java.awt.event.*;
import java.awt.*;
import java.util.*;

import javax.swing.*;

class MainPanel extends Panel implements ButtonsInterface {

	MainPanel() {
		MouseDispatcher mouseDispatcher = new MouseDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);

		doll = new Doll();
	}


	public void initializeButtons(Buttons buttons) {
		buttons.addButton("reset");
	}

	public void buttonPressed(String label) {
		if (label.equals("reset")) {
			for (Limb limb : doll.get_all_limbs())
				limb.relative_angle = 0;
			repaint();
		}
	}

	Doll doll;

	Image buffered_image = null;
	BasicStroke stroke = new BasicStroke(5, BasicStroke.CAP_ROUND,
			BasicStroke.JOIN_ROUND);

	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {

		// double buffering
		if (buffered_image == null)
			buffered_image = createImage(getSize().width, getSize().height);
		Graphics bg = buffered_image.getGraphics();

		// clear
		bg.setColor(Color.white);
		bg.fillRect(0, 0, getSize().width, getSize().height);

		((Graphics2D) bg).setStroke(stroke);
		((Graphics2D) bg).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		doll.paint(bg);

		// double buffering
		bg.dispose();

		g.drawImage(buffered_image, 0, 0, this);

	}

	//
	// central event dispatcher
	//
	public class MouseDispatcher extends MouseAdapter implements
			MouseMotionListener {
		public void mouseMoved(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
			mouseDragged(e);
		}

		public void mouseDragged(MouseEvent e) {
			Point p = e.getPoint();
			
			doll.root_limb.relative_angle = Math.PI * (p.x - 300)/300;
			repaint();
		}

		public void mouseReleased(MouseEvent e) {
		}

		public void mouseClicked(MouseEvent e) {
		}
	}

	public static JFrame frame;

	public static void main(String args[]) {
		frame = new JFrame("Main");
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}

			public void windowOpened(WindowEvent e) {
				frame.repaint();
			}
		});
		frame.addKeyListener(new KeyAdapter() {
			public void keyPress(KeyEvent e) {
				if (e.getKeyCode() == e.VK_C)
					System.exit(0);
			}
		});

		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		MainPanel mainPanel = new MainPanel();
		panel.add("Center", mainPanel);
		panel.add("East", new Buttons(mainPanel));

		frame.add("Center", panel);
		frame.setSize(800, 600);
		frame.setVisible(true);

	}
}
